<h2 class="display-2">Funding Process Update</h2>

<p>Processing of uploaded funding data file <strong>{{filename}}</strong>
was completed successfully, with {{row_count}} funding(s) written
There were {{error_count}} error(s).</p>

<p>You can export a json of the results by clicking this link:
<a href="{{export_url}}">{{export_url}}</a></p>

<p>If you've prefer the file in an alternative format, just visit the Hub and navigate to
Affiliations&gt;Uploaded Tasks and view this task. From this page, you can select an export
from any of a wide range of data formats</p>
